import Helper from '@ember/component/helper';
import { get } from '@ember/object';
import { inject as service } from '@ember/service';
import { assert } from '@ember/debug';
import { isPresent } from '@ember/utils';

function normalizeRouteName(routeName) {
  /* The helper accepts route names in the standard format, ie. my.route.path,
     whereas the `recognize` methods expects the path format, ie.
     /my/route/path. This function normalizes route names to the path format. */
  let normalized = routeName.replace(/\./g, '/');

  if (!normalized.startsWith('/')) {
    normalized = `/${normalized}`;
  }

  if (normalized.endsWith('/index')) {
    normalized = normalized.substring(0, normalized.length - '/index'.length);
  }

  return normalized;
}

export default Helper.extend({
  router: service(),

  compute([routeName]) {
    assert('route-exists: Must provide a routeName', isPresent(routeName));

    return this.routeExists(normalizeRouteName(routeName));
  },

  routeExists(routeName) {
    let router = get(this, 'router');

    if (router.recognize) {
      return router.recognize(routeName) !== null;
    }

    /* For Ember versions below 3.6, the RouterService doesn't have the
       `recognize` function. So if that function doesn't exist, we assume that
       Ember is < 3.6, and instead try to get the function in a more hacky
       way. */

    let recognizer = get(router, '_router._routerMicrolib.recognizer');

    // `recognize` in Ember <3.6 returns undefined instead of null when it fails
    return recognizer.recognize(routeName) !== undefined;
  }
});
